###################################################################
###  mlbook2_prepare_data.r                                     ###
###                                                             ###
###  This is an R script for preparing data files for examples  ###
###  in various chapters of                                     ###
###  Snijders, Tom A.B., and Bosker, Roel J.                    ###
###  Multilevel Analysis:                                       ###
###  An Introduction to Basic and Advanced Multilevel Modeling, ###
###  second edition.                                            ### 
###  London etc.: Sage Publishers, 2012                         ### 
###                                                             ###
###################################################################

# This script reads the original data set, does some transformations,
# and writes a couple of data files.

# read data
library(foreign)
mlbook2 <- read.spss("MLbook_2nded_total_4106-99.sav",to.data.frame=TRUE)

# define function for transforming missing value codes to NA
makemis <- function (a){
  return (ifelse (a==-99, NA , a))
}

# Any missings recognizable in this coding?
count.na <- function(x){sum(is.na(x))}
sapply(mlbook2,count.na)

# What are the variables?
names(mlbook2)

# Transform missing value codes to NA
mlbook2$langPRET <- makemis(mlbook2$langPRET)
mlbook2$langPOST <- makemis(mlbook2$langPOST)
mlbook2$ses      <- makemis(mlbook2$ses)
mlbook2$IQ_verb  <- makemis(mlbook2$IQ_verb)
mlbook2$IQ_perf  <- makemis(mlbook2$IQ_perf)
mlbook2$mixedgra <- makemis(mlbook2$mixedgra)
mlbook2$Minority <- makemis(mlbook2$Minority)
mlbook2$sex      <- makemis(mlbook2$sex)
mlbook2$repeatgr <- makemis(mlbook2$repeatgr)
mlbook2$aritPRET <- makemis(mlbook2$aritPRET)
mlbook2$aritPOST <- makemis(mlbook2$aritPOST)
mlbook2$meetings <- makemis(mlbook2$meetings)
mlbook2$currmeet <- makemis(mlbook2$currmeet)
mlbook2$homework <- makemis(mlbook2$homework)
mlbook2$percmino <- makemis(mlbook2$percmino)
mlbook2$satiprin <- makemis(mlbook2$satiprin)
mlbook2$natitest <- makemis(mlbook2$natitest)
mlbook2$aritdiff <- makemis(mlbook2$aritdiff)
mlbook2$classsiz <- makemis(mlbook2$classsiz)
mlbook2$schoolSES <- makemis(mlbook2$schoolSES)

# For denomination, "missing" is interpreted as a separate category
recode <- function (a){
  return (ifelse (a==-99, 5 , a))
}

table(mlbook2$denomina)
mlbook2$denomina <- recode(mlbook2$denomina)
table(mlbook2$denomina)

# Attach mlbook2 to have less typing to do.
attach(mlbook2)

# Number of missings now?
apply(mlbook2,2,count.na)

# How to calculate school averages:
# schdata <- aggregate(mlbook2, by = list(mlbook2$schoolnr), mean)
# But even with na.action=na.omit, this produced NA in results
# also if aggregate was applied by variable.
# Therefore a more cumbersome procedure:

# define group mean of variable thevar
themean <- function(thevar,id2){
    temp1 <- cbind(id2,thevar)
    temp1 <- temp1[!is.na(temp1[,2]),]
    temp2 <- aggregate(temp1[,2], by=list(temp1[,1]), FUN=mean)
    return(temp2[,2])
}

# However, it turns out that there are 4 schools 
# where langPOST is missing for all pupils.
# These schools must be dropped.

# define schools where a variable is totally missing: all NA
allna <- function(thevar){
    temp1 <- cbind(mlbook2$schoolnr,thevar)
    temp1[,2] <- ifelse(is.na(temp1[,2]),1,0)
    temp2 <- aggregate(temp1[,2], by=list(temp1[,1]), FUN=min)
    temp2[,2] = (temp2[,2]==1)
    return(temp2)
}

nolangPOST <- allna(mlbook2$langPOST)
table(nolangPOST[,2])
nolangPRET <- allna(mlbook2$langPRET)
table(nolangPRET[,2])

# Construct function to disaggregate data frame thevars, 
# where the first variable of thevars is the school number
disaggregate <- function(thevars,id2){
    temp <- merge(id2,thevars,by.x=1,by.y=1)
    return (temp[,2])
}

# Apply this function
nlp <- disaggregate(nolangPOST, mlbook2$schoolnr)
# Take out the schools where langPOST was not collected:
mlbook2r <- mlbook2[!nlp,]

# define group mean in any data set, for level-2 identifier id2
themean <- function(thevar,id2){
    temp1 <- cbind(id2,thevar)
    temp1 <- temp1[!is.na(temp1[,2]),]
    temp2 <- aggregate(temp1[,2], by=list(temp1[,1]), FUN=mean)
    return(temp2[,2])
}

# Globally center IQ_verb, IQ_perf, ses, 
miq <- mean(mlbook2r$IQ_verb,na.rm=T)
mlbook2r$IQ_verb <- mlbook2r$IQ_verb - miq
mlbook2r$IQ_perf <- mlbook2r$IQ_perf - mean(mlbook2r$IQ_perf,na.rm=T)
mlbook2r$ses <- mlbook2r$ses - mean(mlbook2r$ses,na.rm=T)


# calculate group sizes
one <- 0*mlbook2$schoolnr + 1
thesum <- function(thevar,id2){
    temp1 <- cbind(id2,thevar)
    temp1 <- temp1[!is.na(temp1[,2]),]
    temp2 <- aggregate(temp1[,2], by=list(temp1[,1]), FUN=sum)
    return(temp2[,2])
}

schoolsz <- thesum(one[!nlp],mlbook2$schoolnr[!nlp])
length(schoolsz)
table(schoolsz)


# Add school means to data set
iqvmean <- themean(mlbook2r$IQ_verb,mlbook2r$schoolnr)
iqpmean <- themean(mlbook2r$IQ_perf,mlbook2r$schoolnr)
lpmean<- themean(mlbook2r$langPOST,mlbook2r$schoolnr)
armean<- themean(mlbook2r$aritPOST,mlbook2r$schoolnr)
sesmean<- themean(mlbook2r$ses,mlbook2r$schoolnr)
minmean<- themean(mlbook2r$Minority,mlbook2r$schoolnr)
snrmean <- themean(mlbook2r$schoolnr,mlbook2r$schoolnr)
schoolmeans <- data.frame(snrmean,lpmean,armean,iqvmean,iqpmean,sesmean,minmean)
names(schoolmeans) <- c("snrmean","sch_lp","sch_ar","sch_iqv","sch_iqp","sch_ses","sch_min")

# Any missings?
count.na(iqvmean)

# Construct combined data frame
mlbookm <- merge(mlbook2r, schoolmeans, by.x='schoolnr', by.y='snrmean')

detach(mlbook2)

##total students
cat(nrow(mlbookm)," students \n")
# 4016
##total schools
cat(nrow(schoolmeans)," schools \n")
# 211
flush.console()

# How many missings in mlbookm?
apply(mlbookm,2,count.na)

# Construct two data frames with reduced variables.
attach(mlbookm)
# _red for reduced
mlbook_red <- data.frame(schoolnr,pupilNR_new,langPOST,
                       ses,IQ_verb,sex,Minority,
                       denomina,sch_ses,sch_iqv,sch_min)
# _biv for bivariate (its later use in Chapter 16)
mlbook_biv <- data.frame(schoolnr,pupilNR_new,langPOST,aritPOST,
                       ses,IQ_verb,IQ_perf,Minority,
                       denomina,sch_ses,sch_iqv,sch_min)
detach(mlbookm) 

# How many missings in mlbook_red?
apply(mlbook_red,2,count.na)

# Now we are going to prepare the dropping of some missings.
use <- (!is.na(mlbook_red$langPOST))& 
    (!is.na(mlbook_red$ses))& (!is.na(mlbook_red$sex))& (!is.na(mlbook_red$IQ_verb))

sum(!use)
# 258 excluded, 3758 included
# Only use use
mlbook_red <- mlbook_red[use,]

# How many missings now in mlbook_red?
apply(mlbook_red,2,count.na)
dim(mlbook_red)


# And for the other dropping
use_b <- (!is.na(mlbook_biv$ses))& (!is.na(mlbook_biv$IQ_verb)) &
             (!(is.na(mlbook_biv$langPOST)&is.na(mlbook_biv$aritPOST)))
sum(use_b)
sum(!use_b)
apply(mlbook_biv,2,count.na)
# 243 excluded, 3773 included
# Only use use_b
mlbook_biv <- mlbook_biv[use_b,]

# How many missings now in mlbook_biv?
apply(mlbook_biv,2,count.na)
dim(mlbook_biv)

# Another dropping
use_bb <- !is.na(mlbook_biv$langPOST)&!is.na(mlbook_biv$aritPOST)
sum(use_bb)
sum(!use_bb)
mlbook_bb <- mlbook_biv[use_bb,]
dim(mlbook_bb)
apply(mlbook_bb,2,count.na)

# Now we are done, and will write the data files in pretty format.

pretable <- format(mlbook_red,nsmall=2,width=6,digits=2)
write.table(pretable,file="mlbook2_r.dat",row.names=F,quote=F)

pretable <- format(mlbook_biv,nsmall=2,width=6,digits=2)
write.table(pretable,file="mlbook2_b.dat",row.names=F,quote=F)

pretable <- format(mlbook_bb,nsmall=2,width=6,digits=2)
write.table(pretable,file="mlbook2_bb.dat",row.names=F,quote=F)

# Finally construct a data frame that also includes
# missingness as separate variables.
attach(mlbookm)
mlbookmr <- data.frame(schoolnr,pupilNR_new,langPRET,langPOST,
                       ses,IQ_verb,IQ_perf,Minority,
                       denomina,meetings,currmeet,homework,               
                       mixedgra,sch_lp,sch_ses,sch_iqv,sch_iqp,sch_min)
detach(mlbookm)

# Define missingness variables
mis_lp  <-  as.numeric(is.na(mlbookmr$langPOST))
mis_iqv <-  as.numeric(is.na(mlbookmr$IQ_verb))
mis_iqp <-  as.numeric(is.na(mlbookmr$IQ_perf))
mis_ses <-  as.numeric(is.na(mlbookmr$ses))
mis_min <-  as.numeric(is.na(mlbookmr$Minority))

# Also missingness at the school level
fracmis_lp <- aggregate(mis_lp, by = list(mlbookmr$schoolnr), FUN=mean)[,2]
fracmis_iqv <- aggregate(mis_iqv, by = list(mlbookmr$schoolnr), FUN=mean)[,2]
fracmis_iqp <- aggregate(mis_iqp, by = list(mlbookmr$schoolnr), FUN=mean)[,2]
fracmis_ses <- aggregate(mis_ses, by = list(mlbookmr$schoolnr), FUN=mean)[,2]
fracmis_min <- aggregate(mis_min, by = list(mlbookmr$schoolnr), FUN=mean)[,2]
sch_manymiss <- is.na(mlbookmr$meetings) + is.na(mlbookmr$currmeet) +
                  is.na(mlbookmr$homework)

# A descriptive
mean(fracmis_iqp)

# replace level-2 identifier by 1..211
attach(mlbookmr)
uninrs <- unique(schoolnr)
# put group identifiers in original places, if there are any missings
id2 <- match(schoolnr,uninrs)                

# The data frame with missing information is
mlbookmm <- data.frame(id2,pupilNR_new,langPRET,langPOST,ses,IQ_verb,IQ_perf,Minority,
            denomina,meetings,currmeet,homework,sch_manymiss,
            mixedgra,sch_lp,sch_ses,sch_iqv,sch_iqp,sch_min,
            mis_lp,mis_iqv,mis_ses,mis_min)
# This is the data set used for Example 9.3.
apply(mlbookmm,2,count.na)
detach(mlbookmr)

pretable <- format(mlbookmm,nsmall=3,width=7,digits=3)
write.table(pretable,file="mlbook2_mm.dat",row.names=F,quote=F)

